; install.scm
; $Id: install.scm,v 1.19 2006/03/22 14:05:40 inger Exp $

;;------------------------------------------------------------------------------
;; usage sample:
;;(define *pkgs* (list "gdm"))
;;
;;(define (run-install packages)
;;  (let ((error (document:popup '/packages/install
;;                               'packages packages
;;                               'quit-on-destroy #t)))
;;    (or (and (pair? error)
;;             (run-dialog `("/packages/error" error ,error)))
;;      error)))
;;
;;(button "New" (when clicked (format #t "result=~S~%" (run-install *pkgs*))))
;;------------------------------------------------------------------------------

(document:surround "/std/base")
(document:envelop with-translation _ "alterator-packages")

width 400
height 200
title (_ "Applying changes...")

margin 5
spacing 5

(define *packages-progress-error* (make-cell '()))

(define (packages-install-progress args)
  (let ((command (car args))
        (arg (cadr args))
        (rest (if (pair? (cddr args)) (caddr args) "")))
    (cond
     ;; progress begin
     ((and (string=? command "progress")
           (string=? arg "begin"))
      (packages-install-label-id text (_ "Preparing...")))

     ;; start of download/install/whatever
     ((and (string=? command "progress")
           (or (string=? arg "download")
               (string=? arg "install")))
      (packages-install-label-id
       text (if (string=? arg "install") ( _"Install") (_ "Download")))
      (packages-install-bar-id maximum (string->number rest))
      (packages-install-bar-id value 0)
      (packages-install-text-id text ""))

     ;; in progress
     ((string=? command "step")
      (let ((val (+ (string->number rest)
                    (packages-install-bar-id value))))
        (packages-install-text-id append-text arg)
        (packages-install-bar-id value val)))

     ;; error messages
     ((string=? command "error")
      (cell-set! *packages-progress-error*
                 (acons arg rest
                        (cell-ref *packages-progress-error*))))

     ;; normal progress end
     ((and (string=? command "progress")
           (string=? arg "end"))
      (document-exit (or (null? (cell-ref *packages-progress-error*))
                         (cell-ref *packages-progress-error*))))

     ;; progress abort -- sources weren't available
     ((and (string=? command "progress")
           (string=? arg "abort"))
      (document-exit #f))

     ;; not reached normally
     (else
      (packages-install-text-id
       append-text (string-append "XXX" command ":" arg "\n"))))))

(define (packages-extract-label item)
  (and (pair? item)
       (let* ((cd (->string (car item)))
              (from (string-index cd #\[))
              (to (string-index cd #\])))
         (and from to (substring cd (+ from 1) to)))))

(define (packages-install-ensure-sources)
  (let ((cdrom (packages-extract-label
                (find (lambda(item)
                        (string-prefix? "cdrom" (symbol->string (car item))))
                      (plist->alist
                       (cdr (woo-first-command
                             (woo-read "/packages/sources/active"))))))))
    (or (not cdrom)
        (let ((current
               (woo-catch
                (lambda()
                  (woo-get-option
                   (woo-first-command (woo-read "/packages/cdrom/ident")) 'label))
                (lambda(reason) ""))))
          (or (string=? cdrom current)
              (and (eqv? 'ok
                         (document:popup-information
                          (string-append "Please insert cdrom, labeled as " cdrom) 'ok 'cancel))
                   (packages-install-ensure-sources)))))))

(define (packages-install-available packages)
  (filter
   (lambda (e)
     (string=? "available"
               (woo-catch
                (lambda ()
                  (woo-get-option
                   (woo-first-command
                    (woo-read (string-append "/packages/state/" e))) 'state))
                (lambda () ""))))
   packages))

(define (document-exit-1 value)
  (and (global 'quit-on-destroy)
       (woo-write "/packages/control" 'control "quit"))
  (document:end value))

(define (document-exit value)
  (remove-mailbox packages-install-progress)
  (document-exit-1 value)
  (simple-notify document:root 'action "retry"))

(define (packages-install)
  (if (packages-install-ensure-sources)
      (let* ((packages (global 'packages))
             (available
              (or (and (pair? packages)
                       (packages-install-available packages))
                  '())))
        (map
         (lambda (e)
           (woo-write
            (string-append "/packages/state/" e) 'state 'install))
         available)
        (if (and packages (null? available))
            (document-exit-1 #t)
            (begin (woo-write "/packages/control" 'control "commit")
                   (add-mailbox packages-install-progress))))
      (document-exit-1 #f)))

children-align center
(document:id packages-install-label-id
             (label (_ "Applying changes...")))
(document:id packages-install-text-id
             (textbox ""
                      layout-policy 100 -2
                      alterability #f))
(document:id packages-install-bar-id
             (progressbar layout-policy 100 -1
                          maximum 1
                          value 0))
(document:root
 (when loaded (packages-install)))
