; sources
; $Id: sources.scm,v 1.11 2006/03/24 13:59:35 inger Exp $

(document:surround "/std/frame")
(document:envelop with-translation _ "alterator-packages")

(define *packages-sources-title* (_ "Packages Sources"))

;;----------------------------------------------------------------------------
;;

(define (packages-uri-mangle uri)
  (string-join (string-split uri #\:) "#" 'infix))

(define (packages-uri-demangle uri)
  (string-join (string-split uri #\#) ":" 'infix))

(define (packages-exchange-pair item)
  (cons (cdr item) (car item)))

;;----------------------------------------------------------------------------
;;

(define *packages-cdrom-mountpoint*
   (woo-get-option (woo-first-command
                    (woo-read "/packages/cdrom/mountpoint"))
                   'mountpoint))


(define *packages-vendors*
  (acons (_ "<not signed>") ""
         (map packages-exchange-pair
              (plist->alist
               (cdr (woo-first-command
                     (woo-read "/packages/sources/vendors")))))))


(define (packages-sources-convert elt state)
  (let* ((uri (packages-uri-demangle (symbol->string (car elt))))
         (val-list (string-cut (cdr elt) #\|))
         (type (car val-list))
         (key (cond-cadr val-list)))
    (cons uri
          `((,(sure-symbol type). ,state)
            (proto . ,(sure-symbol (car (string-cut uri #\:))))
            ,@(if (not-empty-string? key) `((key . ,(sure-symbol key))) '())))))

(define (packages-sources-read-1 state)
  (map (lambda (elt) (packages-sources-convert elt state))
       (plist->alist
        (cdr
         (woo-first-command
          (woo-read
           (string-append "/packages/sources/" (->string state))))))))

(define (packages-sources-read)
  (sort
   (append (packages-sources-read-1 'active)
           (packages-sources-read-1 'passive))
   (lambda(x y)
     (string<? (car x) (car y)))))

(define (packages-sources-join lst)
  (let loop ((lst lst)
             (result '()))
    (if (null? lst) result
        (let ((first (car lst))
              (next (or (cond-cadr lst) '(""))))
          (if (string=? (car first) (car next))
              (loop (cddr lst) (append1 result
                                        (cons (car first)
                                              (append (cdr first) (cdr next)))))
              (loop (cdr lst) (append1 result first)))))))

(define *packages-sources* (make-cell (acons (_ "<New entry>") '((rpm . passive)
                                                                 (rpm-src . passive)
                                                                 (key . ""))
                                             (packages-sources-join (packages-sources-read)))))

;;----------------------------------------------------------------------------
;;



(define (packages-uri-set! uri key value)
  (cell-set! *packages-sources*
         (assoc-set! (cell-ref *packages-sources*) uri
                    (assoc-set! (assoc-ref (cell-ref *packages-sources*) uri) key value))))

(define (packages-uri-get uri key)
  (assoc-ref (assoc-ref (cell-ref *packages-sources*) uri) key))

(define (packages-uri-key->nth key)
  (list-index ;; DANGER srfi-1
   (lambda (e) (eq? key e))
   (map cdr *packages-vendors*)))

(define (packages-uri-sig->key sig)
  (assoc-ref *packages-vendors* sig))


(define (packages-uri-loc-nth loc)
  (list-index ;; DANGER srfi-1
   (lambda (e) (equal? loc e))
   (map car (cell-ref *packages-sources*))))

(define (packages-uri-pixmap uri)
  (string-append
   "packages/"
   (if (or (eq? (assoc-ref uri 'rpm) 'active)
           (eq? (assoc-ref uri 'rpm-src) 'active))
       "active"
       "passive")
   ".png"))

(define (packages-sources-items)
  (map
   (lambda(elt)
     (cons (car elt) (packages-uri-pixmap (cdr elt))))
   (cell-ref *packages-sources*)))

;;----------------------------------------------------------------------------
;; event handlers

(define (packages-uri-update)
  (let* ((uri (packages-sources text))
         (elt (assoc-ref (cell-ref *packages-sources*) uri))
         (old-entry (packages-uri-get uri 'proto)))

    (packages-uri-cont activity (symbol? old-entry))
    (packages-uri-bin state (and elt (eq? (assoc-ref elt 'rpm) 'active)))
    (packages-uri-src state (and elt (eq? (assoc-ref elt 'rpm-src) 'active)))

    (packages-uri-sig activity (and old-entry (not (eq? old-entry 'cdrom))))
    (packages-uri-sig-combo current (or (and elt (packages-uri-key->nth (assoc-ref elt 'key))) 0))

    (packages-uri-loc-edit text (or (and old-entry uri) ""))
    (packages-uri-loc-but text (or (and old-entry (_ "Replace")) (_ "Add")))

    (packages-uri-cd activity (or (not old-entry) (eq? old-entry 'cdrom)))))

(define (packages-uri-toggle w what)
  (let ((uri (packages-sources text))
        (state (w state))
        (key (packages-uri-sig->key (packages-uri-sig-combo text))))
    ;;(format (current-error-port) "what: ~S: state: ~S key: ~S uri: ~S\n" what state key uri)
    (packages-uri-set! uri what (or (and state 'active) 'passive))
    (packages-sources
     row-item
     (packages-uri-loc-nth uri)
     `(#f . ,(packages-uri-pixmap (assoc-ref (cell-ref *packages-sources*) uri))))
    (woo-write
     (string-append "/packages/sources/" (or (and state "active") "passive"))
     'source (string-append (symbol->string what) "|" (->string key) "|" (packages-uri-mangle uri)))))

(define (packages-uri-signature)
  (let* ((uri (packages-sources text))
         (mangled-uri (packages-uri-mangle uri))
         (oldkey (or (packages-uri-get uri 'key) ""))
         (newkey (packages-uri-sig->key (packages-uri-sig-combo text))))
    ;;(format (current-error-port) "old key: ~S new key: ~S\n" oldkey newkey)
    (or (equal? oldkey newkey)
        (begin
          (packages-uri-set! uri 'key newkey)
          (for-each
           (lambda (w what)
             ;; (format (current-error-port) "do ~S with ~S\n" w what)
             (and (w state)
                  (for-each
                   (lambda (dest key)
                     (woo-write (string-append "/packages/sources/" dest) 'source
                                (string-append what "|" (->string key) "|" mangled-uri)))
                   '("passive" "active") `(,oldkey ,newkey))))
           `(,packages-uri-bin ,packages-uri-src) '("rpm" "rpm-src"))))))


(define (packages-uri-location)
  (let ((olduri (packages-sources text))
        (newuri (packages-uri-loc-edit text)))
    ;;(format (current-error-port) "old uri: ~S\nnew uri: ~S\n" olduri newuri)
    ;; TODO validate newuri
    (if (packages-uri-get olduri 'proto)
        (let ((key (or (packages-uri-get olduri 'key) "")))
          (for-each
           (lambda (w what)
             ;; (format (current-error-port) "do ~S with ~S\n" w what)
             (and (w state)
                  (for-each
                   (lambda (dest uri)
                     (woo-write (string-append "/packages/sources/" dest) 'source
                                (string-append what "|" (->string key) "|" (packages-uri-mangle uri))))
                   '("passive" "active") `(,olduri ,newuri))))
           `(,packages-uri-bin ,packages-uri-src) '("rpm" "rpm-src"))
          (let ((nth (packages-uri-loc-nth olduri)))
            (packages-sources row-item nth newuri)
            (list-set! (cell-ref *packages-sources*) nth
                       (cons newuri (assoc-ref (cell-ref *packages-sources*) olduri)))))
        (let* ((proto (string->symbol (car (string-split newuri #\:))))
               (type (and (eq? proto 'cdrom)
                          (woo-get-option
                           (woo-first-command
                            (woo-read "/packages/cdrom/add")) 'type)))
               (rpm-active (or (and (equal? type "rpm") 'active) 'passive))
               (rpm-src-active (or (and (equal? type "rpm-src") 'active) 'passive)))
          (cell-set! *packages-sources*
                (cons (car (cell-ref *packages-sources*))
                      (acons newuri `((rpm . ,rpm-active)
                                      (rpm-src . ,rpm-src-active)
                                      (key . "")
                                      (proto . ,proto))
                             (cdr (cell-ref *packages-sources*)))))
          ;;(pretty-print *packages-sources* (current-error-port))
          (packages-sources rows (packages-sources-items))
          (packages-sources current 1)))))

(define (packages-uri-cdrom)
  (let ((uri (packages-uri-cdrom-ensure)))
    (and uri
         (packages-uri-loc-edit text uri))))

(define (packages-uri-cdrom-ensure)
  (or (let ((uri (woo-get-option
                  (woo-first-command
                   (woo-read "/packages/cdrom/info")) 'uri)))
        (and (not (equal? "" uri))
             (packages-uri-demangle uri)))
      (and (document:popup '/packages/cdrom)
           (packages-uri-cdrom-ensure))))

(define (packages-uri-mountpoint)
  (let ((mountpoint (packages-uri-cd-mnt text)))
    (woo-write "/packages/cdrom/mountpoint" 'mountpoint mountpoint)))

;;----------------------------------------------------------------------------
;; UI

(groupbox (_ "Packages Sources")
          (layout-policy 100 -2)
          (document:id packages-sources
                       (listbox
                        (layout-policy 100 100)
                        rows (packages-sources-items)
                        (when selected (packages-uri-update)))))
(groupbox (_ "Properties")
          (layout-policy 100 -1)
          (vbox
           (layout-policy 100 100)
           (hbox
            (layout-policy 100 -1)
            spacing 5
            (document:id packages-uri-cont
                         (groupbox (_ "Contains")
                                   (layout-policy -2 -1)
                                   (hbox
                                    (document:id packages-uri-bin
                                                 (checkbox (_ "binary packages") state #f
                                                           (when toggled (packages-uri-toggle packages-uri-bin 'rpm))))
                                    (document:id packages-uri-src
                                                 (checkbox (_ "source packages") state #f
                                                           (when toggled (packages-uri-toggle packages-uri-src 'rpm-src)))))))

            (document:id packages-uri-sig
                         (groupbox (_ "Signed by")
                                   (layout-policy -2 -1)
                                   (document:id packages-uri-sig-combo
                                                (combobox
                                                 (layout-policy 100 -1)
                                                 rows (map car *packages-vendors*)
                               (when selected (packages-uri-signature)))))))
           (groupbox (_ "URI")
                     (layout-policy 100 -1)
                     (hbox
                      (layout-policy 100 -1)
                      spacing 5
                      (document:id packages-uri-loc-edit
                                   (edit ""
                                         (layout-policy 50 -1)
                                         (when return-pressed (packages-uri-location))))
                      (document:id packages-uri-loc-but
                                   (button "" (when clicked (packages-uri-location))))))

           (document:id packages-uri-cd
                        (groupbox "CD/DVD"
                                  activity #f
                                  (layout-policy 100 -1)
                                  (hbox
                                   (layout-policy 100 -1)
                                   spacing 5
                                   (label (_ "Mountpoint:"))
                                   (document:id packages-uri-cd-mnt
                                                (edit
                                                 *packages-cdrom-mountpoint*
                                                 (layout-policy 50 -1)
                                                 (when return-pressed (packages-uri-mountpoint))))
                                   (document:id packages-uri-cd-ident
                                                (button (_ "Identify")
                                                        (when clicked (packages-uri-cdrom)))))))))
