; packages
; $Id: packages.scm,v 1.89 2006/05/04 14:01:30 inger Exp $

(document:surround "/std/frame")
(document:insert "/std/functions")
(document:envelop with-translation _ "alterator-packages")

(document:envelop with-init-attributes (orientation))
(document:envelop with-attributes
                  (entry-pixmap entry-label entry-value
                                packages-tree-items packages-translation packages-tree-text))

(document:envelop with-container-presentations
                  ((packages-status-entry '/packages/status-entry
                                          entry-pixmap
                                          entry-label
                                          entry-value)
                   (packages-tree '/packages/tree)))

(define (tr-string<? x y) (string<? x y))

(define *packages* (_ "Packages Installation"))

;;-------------------------------------------------------------------------------
;; convenience utils
;;-------------------------------------------------------------------------------
(define (packages-pixmap fname)
  (string-append "packages/" fname ".png"))

(define (packages-with-splash thunk)
  (splash-message (_ "Please wait..."))
  (document:release)
  (thunk)
  (splash-message ""))

(define (packages-first-leaf lst)
  (let loop ((lst lst)
             (result '(0)))
    (if (pair? (cond-cadr lst))
        (loop (cadr lst) (append1 result 0))
        result)))

;;-------------------------------------------------------------------------------
;; transaction logging/replaying
;;-------------------------------------------------------------------------------
(document:insert "/packages/log")

;;-------------------------------------------------------------------------------
;; view methods
;;-------------------------------------------------------------------------------
(define *packages-view-alist*
  (filter pair?
          (list
           (last (document:insert "/packages/view-profile"))
           (last (document:insert "/packages/view-groups"))
           (last (document:insert "/packages/view-pending"))
           (last (document:insert "/packages/view-search")))))

(define *packages-view* (make-cell (caar *packages-view-alist*)))

(define (packages-view->this)
  (assv-ref *packages-view-alist* (cell-ref *packages-view*)))

(define (packages-view->label)
  (assv-ref (packages-view->this) 'label))

(define (packages-view->type)
  (assv-ref (packages-view->this) 'type))

(define (packages-view->add major)
  ((assv-ref (packages-view->this) 'add) (or major "")))

(define (packages-view->major)
  ((assv-ref (packages-view->this) 'major)))

(define (packages-view->minor minor)
  ((assv-ref (packages-view->this) 'minor) (or minor "")))

(define (packages-elt-version elt)
  ((assv-ref (packages-view->this) 'elt-version) (or elt "")))

(define (packages-elt-summary elt)
  ((assv-ref (packages-view->this) 'elt-summary) (or elt "")))

(define (packages-elt-description elt)
  ((assv-ref (packages-view->this) 'elt-description) (or elt "")))

(define (packages-elt-state elt)
  (packages-log-add
   ((assv-ref (packages-view->this) 'elt-state) (or elt ""))))

;;===============================================================================
;; UI
;;===============================================================================

;;-------------------------------------------------------------------------------
;; view selector
;;-------------------------------------------------------------------------------
(define (packages-view-update)
  (let ((view (packages-view-id text)))
    (for-each
     (lambda (e)
       (and (string=? view (assv-ref (cdr e) 'label))
            (cell-set! *packages-view* (car e))))
     *packages-view-alist*))
  (packages-search-update)
  (packages-major-update));note: major-update must be after search update to correct auto-translator setup

(define (packages-view-contents)
  (map (lambda (e) (assv-ref (cdr e) 'label))
       *packages-view-alist*))

(define (packages-view-ui)
  (hbox
   layout-policy 100 -1
   (label (_ "View:"))
   (document:id
    packages-view-id
    (combobox
     layout-policy -2 -1
     rows (packages-view-contents)
     current 0
     (when selected (packages-view-update))))))

;;-------------------------------------------------------------------------------
;; search
;;-------------------------------------------------------------------------------
(define (packages-search)
  (packages-view->add (packages-search-keyword-id text))
  (packages-major-update))

(define (packages-search-update)
  (let ((under-search? (procedure? (assv-ref (packages-view->this) 'add))))
    (packages-major-id packages-translation (not under-search?))
    (packages-search-id visibility under-search?)))

(define (packages-search-ui)
  (document:id
   packages-search-id
   (hbox
    layout-policy 100 -1
    visibility (procedure? (assv-ref (packages-view->this) 'add))
    (label (_ "Keywords:"))
    (document:id
     packages-search-keyword-id
     (edit "" (layout-policy -2 -1)
           (when return-pressed
             (packages-with-splash packages-search)))))))

;;-------------------------------------------------------------------------------
;; major
;;-------------------------------------------------------------------------------

(define (packages-major-contents)
  (let ((contents (sort (packages-view->major) tr-string<?)))
    (rows->tree-items (map (lambda(x) (string-cut x #\|)) contents))))

(define (packages-major-update)
  (let ((contents (packages-major-contents)))
    (packages-major-id packages-tree-items contents)
    (and (pair? contents)
         (packages-major-id current (packages-first-leaf contents))))
  (packages-major-id selected))

(define (packages-major-ui)
  (document:id
   packages-major-id
   (packages-tree
    (layout-policy 100 -2)
    packages-translation #t
    (let ((contents (packages-major-contents)))
      (list (packages-tree-items contents)
            (and (pair? contents)
                 (current (packages-first-leaf contents)))))
    (when selected
      (if  (or (null? (packages-major-id tree-items))
               (tree-leaf? (packages-major-id tree-items)
                           (packages-major-id current)))
           (packages-with-splash packages-minor-update))))))

;;-------------------------------------------------------------------------------
;; minor
;;-------------------------------------------------------------------------------
(define (packages-minor-contents)
  (let* ((major (string-join (packages-major-id packages-tree-text) "|"))
         (contents (packages-view->minor major)))
    (map
     (lambda (elt)
       (cons (car elt) (packages-pixmap (cdr elt)))) contents)))

(define (packages-minor-update)
  (let ((contents (packages-minor-contents)))
    (packages-minor-id rows contents)
    (and (pair? contents) (packages-minor-id current 0)))
  (packages-minor-id selected))

(define (packages-minor-state)
  (packages-elt-state (packages-minor-id text))
  (packages-minor-state-1))

(define (packages-minor-state-1)
  (and (eqv? (packages-view->type) 'dynamic)
       (let ((idx (packages-major-id current)))
         (packages-major-id packages-tree-items (packages-major-contents))
         (packages-major-id current idx)))
  (let ((idx (packages-minor-id current)))
    (packages-minor-id rows (packages-minor-contents))
    (packages-minor-id current idx))
  (packages-status-update))

(define (packages-minor-ui)
  (document:id
   packages-minor-id
   (listbox
    layout-policy 100 -2
    (let ((contents (packages-minor-contents)))
      (list rows contents
            (and (pair? contents) (current 0))))
    (when selected (packages-info-update))
    (when double-clicked (packages-with-splash packages-minor-state)))))

;-------------------------------------------------------------------------------
; info
;-------------------------------------------------------------------------------
(define (packages-info-update)
  (let ((elt (packages-minor-id text))
        (state (packages-info-toggle-id state)))
    (if state
        (packages-info-description-id text (packages-elt-description elt))
        (packages-info-summary-id text (packages-elt-summary elt)))))

(define (packages-info-toggle)
  (let ((state (packages-info-toggle-id state)))
    (packages-info-summary-id visibility (not state))
    (packages-info-description-id visibility state)
    (packages-info-update)))

(define (packages-info-toggle-ui)
  (document:id
   packages-info-toggle-id
   (checkbox (_ "Show package description")
             (when toggled (packages-info-toggle)))))

(define (packages-info-ui)
  (hbox
   (document:id packages-info-summary-id (edit "" alterability #f visibility #t))
   (document:id packages-info-description-id (textbox "" alterability #f visibility #f))))

;-------------------------------------------------------------------------------
; status
;-------------------------------------------------------------------------------
(define (packages-status)
  (let ((reply (woo-first-command (woo-read "/packages/control"))))
    (map (lambda (e) (cons e (woo-get-option reply e)))
         '(available installed install upgrade remove keep dsize isize foobaz))))

(define (packages-status-update)
  (let ((val-alist (packages-status)))
    (packages-status-available-id entry-value (assv-ref val-alist 'available))
    (packages-status-installed-id entry-value (assv-ref val-alist 'installed))
    (packages-status-install-id entry-value (assv-ref val-alist 'install))
    (packages-status-upgrade-id entry-value (assv-ref val-alist 'upgrade))
    (packages-status-remove-id entry-value (assv-ref val-alist 'remove))
    (packages-status-keep-id entry-value (assv-ref val-alist 'keep))
    (packages-status-dsize-id entry-value (assv-ref val-alist 'dsize))
    (packages-status-isize-id entry-value (assv-ref val-alist 'isize))

    (packages-status-isize-id
     entry-label
     (or (and (string=? (assv-ref val-alist 'foobaz) "disk")
              (_ "Be freed on disk"))
         (_ "Be used on disk"))))

  (and (global 'frame:next)
       (frame:next activity (packages-log-pending-empty?)))

  ;;(prev-button enabled (packages-log-pending-empty?))

  (packages-command-update-id activity (packages-log-pending-empty?))
  (packages-command-reset-id activity (not (packages-log-pending-empty?)))
  (packages-command-apply-id activity (not (packages-log-pending-empty?)))
  (packages-command-undo-id activity (not (packages-log-undo-empty?)))
  (packages-command-redo-id activity (not (packages-log-redo-empty?))))

(define (packages-status-toggle-ui)
  (document:id
   packages-status-toggle-id
   (checkbox
    (_ "Show statistics")
    (when toggled
      (packages-status-id visibility (packages-status-toggle-id state))))))

(define (packages-status-ui)
  (let ((val-alist (packages-status)))
    (document:id
     packages-status-id
     (vbox visibility #f
           (document:id packages-status-available-id
                        (packages-status-entry (packages-pixmap "available")
                                               (_ "Available")
                                               (assv-ref val-alist 'available)))
           (document:id packages-status-installed-id
                        (packages-status-entry (packages-pixmap "installed")
                                               (_ "Installed")
                                               (assv-ref val-alist 'installed)))
           (document:id packages-status-install-id
                        (packages-status-entry (packages-pixmap "be-installed")
                                               (_ "install")
                                               (assv-ref val-alist 'install)))
           (document:id packages-status-upgrade-id
                        (packages-status-entry (packages-pixmap "be-upgraded")
                                               (_ "upgrade")
                                               (assv-ref val-alist 'upgrade)))
           (document:id packages-status-keep-id
                        (packages-status-entry (packages-pixmap "be-kept")
                                               (_ "keep")
                                               (assv-ref val-alist 'keep)))
           (document:id packages-status-remove-id
                        (packages-status-entry (packages-pixmap "be-removed")
                                               (_ "remove")
                                               (assv-ref val-alist 'remove)))
           (document:id packages-status-dsize-id
                        (packages-status-entry (packages-pixmap "moreinfo")
                                               (_ "Need to get")
                                               (assv-ref val-alist 'dsize)))
           (document:id packages-status-isize-id
                        (packages-status-entry (packages-pixmap "moreinfo")
                                               (_"Be used on disk")
                                               (assv-ref val-alist 'isize)))))))

;;-------------------------------------------------------------------------------
;; command toolbox
;;-------------------------------------------------------------------------------
(define (packages-command-update)
  (let ((result (document:popup '/packages/update)))
    (or (and (pair? result)
             (document:popup '/packages/error 'error result))
        (and (not result)
             (packages-command-reset))))
  (packages-log-reset)
  (packages-view-update)
  (packages-status-update))

(define (packages-command-reset)
  (woo-write "/packages/control" 'control "quit")
  (if (eqv? (packages-view->type) 'dynamic)
      (packages-major-update)
      (packages-minor-update))
  (packages-log-reset)
  (packages-status-update))

(define (packages-command-apply)
  (let ((result (document:popup '/packages/install)))
    (or (and (pair? result)
             (document:popup '/packages/error 'error result))
        (and (not result)
             (packages-command-reset))))
  (packages-log-reset)
  (packages-view-update)
  (packages-status-update))

(define (packages-command-undo)
  (packages-log-undo)
  (packages-minor-state-1))

(define (packages-command-redo)
  (packages-log-redo)
  (packages-minor-state-1))

(define (packages-command-quit)
  (and (not (packages-log-pending-empty?))
       (packages-command-apply))
  (woo-write "/packages/control" 'control "quit"))

(define (packages-command-ui)
  (let* ((values (packages-status))
         (pending
          (pair? (memq #f (map (lambda (e) (string=? "0" (assoc-ref values e)))
                               '(install upgrade remove))))))
    (groupbox
     ""
     orientation 'horizontal
     (document:id packages-command-update-id
                  (button (_ "Update")
                          layout-policy -2 -1
                          activity (not pending)
                          (when clicked (packages-command-update))))

     (document:id packages-command-reset-id
                  (button (_ "Reset")
                          layout-policy -2 -1
                          activity pending
                          (when clicked
                            (packages-with-splash packages-command-reset))))

     (document:id packages-command-undo-id
                  (button (_ "Undo")
                          layout-policy -2 -1
                          activity (not (packages-log-undo-empty?))
                          (when clicked
                            (packages-with-splash packages-command-undo))))

     (document:id packages-command-redo-id
                  (button (_ "Redo")
                          layout-policy -2 -1
                          activity (not (packages-log-redo-empty?))
                          (when clicked
                            (packages-with-splash packages-command-redo))))

     (document:id packages-command-apply-id
                  (button (_ "Apply")
                          layout-policy -2 -1
                          activity pending
                          (when clicked (packages-command-apply)))))))

;;-------------------------------------------------------------------------------
;; main
;;-------------------------------------------------------------------------------
(groupbox
 ""
 (hbox
  (vbox
   (packages-view-ui)
   (packages-major-ui)
   (packages-status-ui)
   (packages-status-toggle-ui))
  (vbox
   (packages-search-ui)
   (packages-minor-ui)
   (packages-info-ui)
   (packages-info-toggle-ui))))

(packages-command-ui)

(and (global 'frame:next-button)
     (frame:next activity (packages-log-pending-empty?)))

(frame:on-next
 (lambda()
   (if (packages-log-pending-empty?)
       (woo-write "/packages/control" 'control "quit")
       'cancel)))

;; (on-apply
;;  (packages-command-quit))

(when loaded
  (packages-info-update))

(document:root
 (when destroyed
   (woo-write "/packages/control" 'control "quit")))
